@@ -3,7 +3,7 @@ package Locale::Maketext::Utils;
 use strict;
 use warnings;
 
-use version;our $VERSION = qv('0.0.2');
+use version;our $VERSION = qv('0.0.3');
 
 use Locale::Maketext;
 use base qw(Locale::Maketext);
@@ -102,13 +102,72 @@ sub fetch {
     return shift->maketext(@_);	
 }
 
+sub lang_names_hashref {
+    my ($lh, @langcodes) = @_;
+    
+    if(!@langcodes) { # they havn't specified any langcodes...
+        require File::Slurp; # only needed here, so we don't use() it
+        require File::Spec;  # only needed here, so we don't use() it
+        
+        my @search;
+        my $path = $lh->get_base_class();
+        $path =~ s{::}{/}g; # !!!! make this File::Spec safe !! File::Spec->seperator() !-e
+        
+        if(ref $lh->{'_lang_pm_search_paths'} eq 'ARRAY') {
+            @search = @{ $lh->{'_lang_pm_search_paths'} };
+        }
+        
+        @search = @INC if !@search; # they havn't told us where they are specifically
+
+        DIR:
+        for my $dir (@search) {
+            my $lookin = File::Spec->catdir($dir, $path);
+            next DIR if !-d $lookin;
+            PM:
+            for my $pm (grep { /^\w+\.pm$/ } File::Slurp::read_dir($lookin)) {
+                $pm =~ s{\.pm$}{};
+                next PM if !$pm;
+                push @langcodes, $pm;
+            }
+        }
+    } 
+    
+    require Locales::Language; # only needed here, so we don't use() it
+    my $obj_two_char = substr($lh->language_tag(), 0, 2); # Locales::Language only does two char ...
+    Locales::Language::setLocale( $obj_two_char );
+    my $langname = {};
+    
+    for my $code ('en', @langcodes) { # en since its "built in"
+        my $two_char = substr($code, 0, 2); # Locales::Language only does two char ...
+        my $left_ovr = length $code > 2 ? uc( substr($code, 3) ) : '';
+        my $long_nam = Locales::Language::code2language( $two_char );
+
+        $langname->{ $code } = $long_nam || $code; 
+        $langname->{ $code } .= " ($left_ovr)" if $left_ovr && $long_nam;
+    }
+    
+    return $langname;
+}
+
+sub loadable_lang_names_hashref {
+    my ($lh, @langcodes) = @_;
+    
+    my $langname = $lh->lang_names_hashref(@langcodes);
+    
+    for my $tag( keys %{ $langname }) {
+        delete $langname->{$tag} if !$lh->langtag_is_loadable( $tag );
+    }
+    
+    return $langname;
+}
+
 1;
 
 __END__
 
 =head1 NAME
 
-Locale::Maketext::Utils - Adds some utility funtionality and failure handling to Local::Maketext handles
+Locale::Maketext::Utils - Adds some utility functionality and failure handling to Local::Maketext handles
 
 =head1 SYNOPSIS
 
@@ -153,6 +212,10 @@ Alias for
 
     $lh->maketext($key, @args);
 
+=head2 $lh->get_base_class()
+
+    Returns the base class of the object. So if $lh is a MyApp::Localize::fr object then it returns MyApp::Localize
+
 =head2 $lh->get_language_tag()
 
 Returns the real language name space being used, not language_tag()'s "cleaned up" one
@@ -163,6 +226,42 @@ Returns 0 if the argument is not a language that can be used to get a handle.
 
 Returns the language handle if it is a language that can be used to get a handle.
 
+=head2 $lh->lang_names_hashref()
+
+This returns a hashref whose keys are the language tags and the values are the 
+name of language tag in $lh's native langauge.
+
+It can be called several ways:
+
+=over 4
+
+=item * Give it a list of tags to lookup
+
+    $lh->lang_names_hashref(@lang_tags)
+
+=item * Have it search @INC for Base/Class/*.pm's 
+
+    $lh->lang_names_hashref() # IE no args
+
+=item * Have it search specific places for Base/Class/*.pm's 
+
+    local $lh->{'_lang_pm_search_paths'} = \@lang_paths; # array ref of directories
+    $lh->lang_names_hashref() # IE no args
+
+=back
+
+The module it uses for lookup (L<Locales::Language>) is only required when this method is called.
+
+The module it uses for lookup (L<Locales::Language>) is currently limited to two character codes but we try to handle it gracefully here.
+
+Does not ensure that the tags are loadable, to do that see below.
+
+=head2 $lh->loadable_lang_names_hashref()
+
+Exactly the same as $lh->lang_names_hashref() (because it calls that method...) except it only contains tags that are loadable.
+
+Has additional overhead of calling $lh->langtag_is_loadable() on each key. So most likely you'd use this on a single specific place (a page to choose their language setting for instance) instead of calling it on every instance your script is run.
+
 =head2 $lh->append_to_lexicons( $lexicons_hashref );
 
 This method allows modules or script to append to the object's Lexicons
@@ -233,7 +332,7 @@ If $lh->{'_get_key_from_lookup'} is not a code ref, or $lh->{'_get_key_from_look
 
 =head1 SEE ALSO
 
-L<Locale::Maketext>
+L<Locale::Maketext>, L<Locales::Language>
 
 =head1 SUGGESTIONS
 
